/* 
 * HmFe (C) is a FEM analysis code. 
 *
 * Copyright (C) 1996-2005
 *
 * Marco Morandini  <morandini@aero.polimi.it>
 * Teodoro Merlini  <merlini@aero.polimi.it>
 *
 * Dipartimento di Ingegneria Aerospaziale - Politecnico di Milano
 * via La Masa, 34 - 20156 Milano, Italy
 * http://www.aero.polimi.it
 *
 * Changing this copyright notice is forbidden.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 */
/*
 * MBDyn (C) is a multibody analysis code. 
 * http://www.mbdyn.org
 *
 * Copyright (C) 1996-2005
 *
 * This code is a partial merge of HmFe and MBDyn.
 *
 * Pierangelo Masarati  <masarati@aero.polimi.it>
 * Paolo Mantegazza     <mantegazza@aero.polimi.it>
 *
 * Dipartimento di Ingegneria Aerospaziale - Politecnico di Milano
 * via La Masa, 34 - 20156 Milano, Italy
 * http://www.aero.polimi.it
 *
 * Changing this copyright notice is forbidden.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation (version 2 of the License).
 * 
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#ifndef Rot_hh
#define Rot_hh

#include <matvecexp.h>

namespace RotManip {

/**
 * Compute the rotation matrix Phi given Euler Rogriguez's parameters phi
 */
template<class T>
Mat3x3T<T> Rot(const Vec3T<T> & phi);

/**
 * Compute G matrix given Euler Rogriguez's parameters Phi
 * G defined in such a way that dPhi * PhiT = G * dphi
 */
template<class T>
 Mat3x3T<T> DRot(const Vec3T<T> & phi);

/**
 * Compute rotation matrix Phi and Ga matrix 
 * given Euler Rogriguez's parameters Phi
 */
template<class T>
void RotAndDRot(const Vec3T<T> & phi, Mat3x3T<T> & Phi, Mat3x3T<T> & Ga);

/**
 * Compute the inverse transpose of G matrix given Euler Rogriguez's parameters Phi
 */
template<class T>
Mat3x3T<T> DRot_IT(const Vec3T<T> & phi);

/**
 * Compute the inverse of G matrix given Euler Rogriguez's parameters Phi
 */
template<class T>
Mat3x3T<T> DRot_I(const Vec3T<T> & phi);

/**
 * Compute inverse transpose ot rotation matrix Phi and Ga matrix 
 * given Euler Rogriguez's parameters Phi
 */
template<class T>
void RotAndDRot_IT(const Vec3T<T> & phi, Mat3x3T<T> & PhiIT, Mat3x3T<T> & GaIT);

/**
 * Compute Euler Rogriguez's parameters phi given rotation matrix Phi
 */
template<class T>
Vec3T<T> VecRot(const Mat3x3T<T> & Phi);

/**
 * Compute, given Euler Rogriguez's parameters phi, L matrix such that
 * dG * a = L(phi, a) * dphi
 */
template<class T>
Mat3x3T<T> Elle(const Vec3T<T> & phi, const Vec3T<T> & a);

} //end of namespace RotManip



namespace RoTrManip {

MatExp Elle(const VecExp & phi, const VecExp & a);

MatExp RoTr(const VecExp & eta);

MatExp DRoTr(const VecExp & eta);

void RoTrAndDRoTr(const VecExp & eta, MatExp & H, MatExp & Th);

MatExp DRoTr_It (const VecExp & eta);

MatExp DRoTr_I (const VecExp & eta);

void RoTrAndDRoTr_It(const VecExp & eta, MatExp & HIt, MatExp & ThIt);

VecExp Helix (const MatExp & H);

} //end of namespace RoTrManip
#endif // Rot_hh

